using System;
using System.Collections.Generic;
using System.Text;

using EllieMae.Encompass.ComponentModel;
using EllieMae.Encompass.Automation;
using EllieMae.Encompass.BusinessObjects;
using EllieMae.Encompass.BusinessObjects.Loans;

using System.IO;
using System.Timers;
using System.Diagnostics;

namespace ExitEncompassAt4AM
{
    [Plugin]
    public class EncompassPlugin
    {
        private Timer timer_exit = null;                    // timer to watch & restart
        private DateTime time_started = DateTime.MinValue;  // time when application started

        public EncompassPlugin()
        {
            EncompassApplication.Login += new EventHandler(EncompassApplication_Login);
        }

        // return TRUE if plugin should be activated
        private bool IsAllowed()
        {
            if (EncompassApplication.Session.UserID == "admin")
            {
                return false; // admin doesn't restart
            }
            else
            {
                return true; // enable for everyone else
            }
        }

        private void EncompassApplication_Login(object sender, EventArgs e)
        {
            if (IsAllowed() && timer_exit == null)
            {
                AddToLog("Starting Timer");
                try
                {
                    time_started = DateTime.Now;
                    timer_exit = new System.Timers.Timer(10000); // check every 10 seconds
                    timer_exit.Elapsed += new ElapsedEventHandler(OnTimer);
                    timer_exit.AutoReset = false;   // we restart timer manually
                    timer_exit.Enabled = true;      // start
                }
                catch (Exception ex)
                {
                    AddToLog("Error: " + ex.Message);
                }
            }
        }

        private void OnTimer(object sender, ElapsedEventArgs e)
        {
            // we want to restart when:
            // 1. time crosses 4AM
            // 2. system returned from standby/hibernation which crossed 4AM
            
            // time now
            DateTime dtNow = DateTime.Now;

            // cutoff time for today
            DateTime dtTodayCutoff = new DateTime(dtNow.Year, dtNow.Month, dtNow.Day, 4, 0, 0);

            // AddToLog("Tick: Now=" + dtNow.ToString("yyyy-MM-dd HH:mm:ss") + 
            //    " Cutoff=" + dtTodayCutoff.ToString("yyyy-MM-dd HH:mm:ss"));

            // compare times against our 4AM cutoff for today
            if (time_started < dtTodayCutoff && dtNow >= dtTodayCutoff)
            {
                try
                {
                    Loan loan = EllieMae.Encompass.Automation.EncompassApplication.CurrentLoan;
                    if (loan != null)
                    {
                        AddToLog("Closing Loan " + loan.LoanNumber + " - " + loan.LoanName);
                        loan.Close();
                    }
                }
                catch (Exception ex)
                {
                    AddToLog("Error: " + ex.Message);
                }

                try
                {
                    AddToLog("Exiting Encompass");
                    Process p = Process.GetCurrentProcess();
                    p.Kill();
                }
                catch (Exception ex)
                {
                    AddToLog("Error: " + ex.Message);
                }
            }
            else
            {
                // start timer
                timer_exit.Enabled = true;
            }
        }

        public static void AddToLog(string s)
        {
            // put your logging code here if needed
            Debug.WriteLine("ExitEncompassAt4AM\t" + DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss") + "\t" + s);  
        }
    }
}
